@@ -48,6 +48,7 @@ using namespace bpp;
 // From the STL:
 #include <ctype.h>
 #include <map>
+#include <iostream>
 
 using namespace std;
 
@@ -56,32 +57,36 @@ using namespace std;
 void AbstractAlphabet::updateMaps_(size_t pos, const AlphabetState& st) {
   if (letters_.find(st.getLetter()) == letters_.end())
     letters_[st.getLetter()] = pos;
+  else
+    throw Exception("AbstractAlphabet::updateMaps_. A state with the same character code already exists! " + st.getLetter() + ".");
   if (nums_.find(st.getNum()) == nums_.end())
     nums_[st.getNum()] = pos;
+  else
+    nums_[st.getNum()] = min(pos, nums_[st.getNum()]);
 }
 
 /******************************************************************************/
 
-void AbstractAlphabet::registerState(const AlphabetState& st) {
+void AbstractAlphabet::registerState(AlphabetState* st) throw (Exception) {
   // Add the state to the vector
-  alphabet_.push_back(st.clone());
+  alphabet_.push_back(st);
   // Update the maps
-  updateMaps_(alphabet_.size(), st);
+  updateMaps_(alphabet_.size() - 1, *st);
 }
 
 /******************************************************************************/
 
-void AbstractAlphabet::setState(size_t pos, const AlphabetState& st)
-  throw (IndexOutOfBoundsException) {
+void AbstractAlphabet::setState(size_t pos, AlphabetState* st)
+  throw (Exception, IndexOutOfBoundsException) {
     if (pos > alphabet_.size())
       throw IndexOutOfBoundsException("AbstractAlphabet::setState: incorect position", pos, 0, alphabet_.size());
     // Delete the state if not empty
     if (alphabet_[pos] != 0)
       delete alphabet_[pos];
     // Put the state in the vector
-    alphabet_[pos] = st.clone();
+    alphabet_[pos] = st;
     // Update the maps
-    updateMaps_(pos, st);
+    updateMaps_(pos, *st);
   }
 
 /******************************************************************************/
@@ -95,8 +100,44 @@ const AlphabetState& AbstractAlphabet::getState(const std::string& letter) const
 
 /******************************************************************************/
 
+size_t AbstractAlphabet::getStateIndex(const std::string& letter) const throw (BadCharException) {
+  map<string, size_t>::const_iterator it = letters_.find(letter);
+  if (it == letters_.end())
+    throw BadCharException(letter, "AbstractAlphabet::getStateIndex(string): Specified base unknown", this);
+  return it->second;
+}
+
+/******************************************************************************/
+
 const AlphabetState& AbstractAlphabet::getState(int num) const throw (BadIntException) {
   map<int, size_t>::const_iterator it = nums_.find(num);
+  if (it == nums_.end())
+    throw BadIntException(num, "AbstractAlphabet::getState(int): Specified base unknown", this);
+  return *(alphabet_[it->second]);
+}
+
+/******************************************************************************/
+
+size_t AbstractAlphabet::getStateIndex(int num) const throw (BadIntException) {
+  map<int, size_t>::const_iterator it = nums_.find(num);
+  if (it == nums_.end())
+    throw BadIntException(num, "AbstractAlphabet::getStateIndex(int): Specified base unknown", this);
+  return it->second;
+}
+
+/******************************************************************************/
+
+AlphabetState& AbstractAlphabet::getState(const std::string& letter) throw (BadCharException) {
+  map<string, size_t>::iterator it = letters_.find(letter);
+  if (it == letters_.end())
+    throw BadCharException(letter, "AbstractAlphabet::getState(string): Specified base unknown", this);
+  return * (alphabet_[it->second]);
+}
+
+/******************************************************************************/
+
+AlphabetState& AbstractAlphabet::getState(int num) throw (BadIntException) {
+  map<int, size_t>::iterator it = nums_.find(num);
   if (it == nums_.end())
     throw BadIntException(num, "AbstractAlphabet::getState(int): Specified base unknown", this);
   return * (alphabet_[it->second]);
@@ -242,11 +283,11 @@ std::string AbstractAlphabet::getGeneric(const std::vector<std::string>& states)
 
 const std::vector<int>& AbstractAlphabet::getSupportedInts() const
 {
-  if(intList_.size() == 0)
+  if(intList_.size() != alphabet_.size())
   {
     intList_.resize(alphabet_.size());
     charList_.resize(alphabet_.size());
-    for(unsigned int i = 0; i < alphabet_.size(); i++)
+    for (size_t i = 0; i < alphabet_.size(); ++i)
     {
       intList_[i]  = alphabet_[i]->getNum();
       charList_[i] = alphabet_[i]->getLetter();
@@ -259,11 +300,11 @@ const std::vector<int>& AbstractAlphabet::getSupportedInts() const
 
 const std::vector<std::string>& AbstractAlphabet::getSupportedChars() const
 {
-  if(charList_.size() == 0)
+  if(charList_.size() != alphabet_.size())
   {
     intList_.resize(alphabet_.size());
     charList_.resize(alphabet_.size());
-    for(unsigned int i = 0; i < alphabet_.size(); i++)
+    for (size_t i = 0; i < alphabet_.size(); ++i)
     {
       intList_[i]  = alphabet_[i]->getNum();
       charList_[i] = alphabet_[i]->getLetter();
