@@ -104,9 +104,35 @@ namespace bpp
 		
     AbstractAlphabet(): alphabet_(), letters_(), nums_(), charList_(), intList_() {}
 
+    AbstractAlphabet(const AbstractAlphabet& alph) : alphabet_(), letters_(alph.letters_), nums_(alph.nums_), charList_(alph.charList_), intList_(alph.intList_)
+    {
+      for (size_t i = 0; i < alph.alphabet_.size(); ++i)
+        alphabet_.push_back(new AlphabetState(*alph.alphabet_[i]));
+    }
+
+    AbstractAlphabet& operator=(const AbstractAlphabet& alph)
+    {
+      for (size_t i = 0 ; i < alphabet_.size() ; ++i)
+        delete alphabet_[i];
+
+      for (size_t i = 0; i < alph.alphabet_.size(); ++i)
+        alphabet_.push_back(new AlphabetState(*alph.alphabet_[i]));
+
+      letters_  = alph.letters_;
+      nums_     = alph.nums_;
+      charList_ = alph.charList_;
+      intList_  = alph.intList_;
+
+      return *this;
+    }
+
+#ifndef NO_VIRTUAL_COV
+    virtual AbstractAlphabet* clone() const = 0;
+#endif
+
     virtual ~AbstractAlphabet()
     {
-      for (unsigned int i = 0 ; i < alphabet_.size() ; i++)
+      for (size_t i = 0 ; i < alphabet_.size() ; ++i)
         delete alphabet_[i];
     }
 	
@@ -116,6 +142,7 @@ namespace bpp
      *
      * @{
      */
+    size_t getNumberOfStates() const { return alphabet_.size(); }
     unsigned int getNumberOfChars() const { return static_cast<unsigned int>(alphabet_.size()); }
     std::string getName(const std::string& state) const throw (BadCharException);
     std::string getName(int state) const throw (BadIntException);
@@ -138,6 +165,28 @@ namespace bpp
      * @name Specific methods to access AlphabetState
      * @{
      */
+    /**
+     * @brief Get a state at a position in the alphabet_ vector.
+     *
+     * This method must be overloaded in specialized classes to send back
+     * a reference of the corect type.
+     *
+     * @param stateIndex The index of the state in the alphabet_ vector.
+     * @throw IndexOutOfBoundsException If the index is invalid.
+     */
+    virtual AlphabetState& getStateAt(size_t stateIndex) throw (IndexOutOfBoundsException);
+    
+    /**
+     * @brief Get a state at a position in the alphabet_ vector.
+     *
+     * This method must be overloaded in specialized classes to send back
+     * a reference of the corect type.
+     *
+     * @param stateIndex The index of the state in the alphabet_ vector.
+     * @throw IndexOutOfBoundsException If the index is invalid.
+     */
+    virtual const AlphabetState& getStateAt(size_t stateIndex) const throw (IndexOutOfBoundsException);
+
     /**
      * @brief Get a state by its letter.
      *
@@ -148,6 +197,9 @@ namespace bpp
      * @throw BadCharException If the letter is not in the Alphabet.
      */
     const AlphabetState& getState(const std::string& letter) const throw (BadCharException);
+
+    AlphabetState& getState(const std::string& letter) throw (BadCharException);
+    
     /**
      * @brief Get a state by its num.
      *
@@ -158,6 +210,20 @@ namespace bpp
      * @throw BadIntException If the num is not in the Alphabet.
      */
     const AlphabetState& getState(int num) const throw (BadIntException);
+
+    AlphabetState& getState(int num) throw (BadIntException);
+
+    int getIntCodeAt(size_t stateIndex) const throw (IndexOutOfBoundsException) {
+      return getStateAt(stateIndex).getNum();
+    }
+
+    const std::string& getCharCodeAt(size_t stateIndex) const throw (IndexOutOfBoundsException) {
+      return getStateAt(stateIndex).getLetter();
+    }
+
+    size_t getStateIndex(int state) const throw (BadIntException);
+    
+    size_t getStateIndex(const std::string& state) const throw (BadCharException);
     /** @} */
 
   protected:
@@ -165,53 +231,43 @@ namespace bpp
      * @brief Add a state to the Alphabet.
      *
      * @param st The state to add.
+     * @throw Exception If a wrong alphabet state is provided.
      */
-    virtual void registerState(const AlphabetState& st);
+    virtual void registerState(AlphabetState* st) throw (Exception);
+    
     /**
      * @brief Set a state in the Alphabet.
      *
      * @param pos The index of the state in the alphabet_ vector.
      * @param st The new state to put in the Alphabet.
+     * @throw Exception If a wrong alphabet state is provided.
+     * @throw IndexOutOfBoundsException If an incorrect index is provided.
      */
-    virtual void setState(size_t pos, const AlphabetState& st) throw (IndexOutOfBoundsException);
+    virtual void setState(size_t pos, AlphabetState* st) throw (Exception, IndexOutOfBoundsException);
+    
     /**
      * @brief Resize the private alphabet_ vector.
      *
      * @param size The new size of the Alphabet.
      */
-    void resize(unsigned int size) { alphabet_.resize(size); }
-    /**
-     * @brief Get a state at a position in the alphabet_ vector.
-     *
-     * This method must be overloaded in specialized classes to send back
-     * a reference of the corect type.
-     *
-     * @param pos The index of the state in the alphabet_ vector.
-     * @throw IndexOutOfBoundsException If pos is out of the vector.
-     */
-    virtual AlphabetState& getStateAt(size_t pos) throw (IndexOutOfBoundsException);
-    /**
-     * @brief Get a state at a position in the alphabet_ vector.
-     *
-     * This method must be overloaded in specialized classes to send back
-     * a reference of the corect type.
-     *
-     * @param pos The index of the state in the alphabet_ vector.
-     * @throw IndexOutOfBoundsException If pos is out of the vector.
-     */
-    virtual const AlphabetState& getStateAt(size_t pos) const throw (IndexOutOfBoundsException);
-
+    void resize(size_t size) { alphabet_.resize(size); }
+    
     /**
      * @brief Re-update the maps using the alphabet_ vector content.
      */
     void remap() {
-      for (size_t i = 0 ; i < alphabet_.size() ; i++) {
-        updateMaps_(i, * alphabet_[i]);
+      letters_.clear();
+      nums_.clear();
+      for (size_t i = 0; i < alphabet_.size(); ++i) {
+        updateMaps_(i, *alphabet_[i]);
       }
     }
 
     unsigned int getStateCodingSize() const { return 1; }
 
+    bool equals(const Alphabet& alphabet) const {
+      return getAlphabetType() == alphabet.getAlphabetType();
+    }
   };
 
 } //end of namespace bpp.
