@@ -47,6 +47,8 @@ knowledge of the CeCILL license and that you accept its terms.
 #include "AlphabetExceptions.h"
 #include "AlphabetState.h"
 
+#include <Bpp/Clonable.h>
+
 /**
  * @mainpage
  *
@@ -112,18 +114,37 @@ namespace bpp
  * the Alphabet classes, and the two methods intToChar() and charToInt().
  * The Alphabet interface also provides other methods, like getting the full name
  * of the states and so on.
+ *
+ * An Alphabet object in itself stores the states as AlphabetStates object, in a
+ * potentially arbitrary but consistent series. All states are then indexed from
+ * 0 to 'numbersOfChars'. The number of states is equal to the number of string
+ * representations, but is usually higher than the number of int representation,
+ * as several characters can correspond to the same state (for instance X, N and ?
+ * in nucleotide alphabets).
  * 
  * The alphabet objects may throw several exceptions derived of the AlphabetException
  * class.
  *
  * @see AlphabetException, BadCharException, BadIntException 
  */
-class Alphabet
+  class Alphabet:
+    public virtual Clonable
 {
   public:
     Alphabet() {}
     virtual ~Alphabet() {}
+
+  /**
+   * @name The Clonable interface
+   *
+   * @{
+   */
+#ifndef NO_VIRTUAL_COV
+  Alphabet* clone() const = 0;
+#endif
   
+  /** @} */
+
   public:
     
     /**
@@ -151,7 +172,29 @@ class Alphabet
     virtual std::string getName(const std::string& state) const throw (BadCharException) = 0;
 
     /**
-     * @name = Tests
+     * @return The int code of a given state.
+     * @param stateIndex The index of the state to fetch.
+     */
+    virtual int getIntCodeAt(size_t stateIndex) const throw (IndexOutOfBoundsException) = 0;
+
+    /**
+     * @return The char code of a given state.
+     * @param stateIndex The index of the state to fetch.
+     */
+    virtual const std::string& getCharCodeAt(size_t stateIndex) const throw (IndexOutOfBoundsException) = 0;
+
+    /**
+     * @return The indices of the states with corresponding int code.
+     */
+    virtual size_t getStateIndex(int state) const throw (BadIntException) = 0;
+    
+    /**
+     * @return The index of the state with corresponding char code.
+     */
+    virtual size_t getStateIndex(const std::string& state) const throw (BadCharException) = 0;
+
+    /**
+     * @name Tests
      *
      * @{
      */
@@ -181,9 +224,20 @@ class Alphabet
      * @{
      */
 
+    /**
+     * @brief Get a state given its index.
+     *
+     * @param stateIndex The index of the state.
+     * @return The AlphabetState.
+     * @throw IndexOutOfBoundsException When index is not a valid.
+     */
+    virtual const AlphabetState& getStateAt(size_t stateIndex) const throw (IndexOutOfBoundsException) = 0;
+
     /**
      * @brief Get a state given its int description.
      *
+     * Note: several states can share the same int values. This function will return one.
+     *
      * @param state The int description.
      * @return The AlphabetState.
      * @throw BadIntException When state is not a valid integer.
@@ -231,7 +285,15 @@ class Alphabet
      *
      * @{
      */
-    
+ 
+    /**
+     * @brief This is a convenient  alias for getNumberOfChars(), returning a size_t
+     * instead of unsigned int.
+     *
+     * This funcion is typically used il loops over all states of an alphabet.
+     */
+    virtual size_t getNumberOfStates() const = 0;
+   
     /**
      * @brief Get the number of supported characters in this alphabet,
      * including generic characters (e.g. return 20 for DNA alphabet).
@@ -382,6 +444,13 @@ class Alphabet
      * @author Sylvain Gaillard
      */
     virtual unsigned int getStateCodingSize() const = 0;
+
+    /**
+     * @brief Comparison of alphabets
+     *
+     * @return true If the two instances are of the same class.
+     */
+    virtual bool equals(const Alphabet& alphabet) const = 0;
 };
 
 } //end of namespace bpp.
