@@ -5,7 +5,7 @@
 //
 
 /*
-   Copyright or  or Copr. CNRS, (November 17, 2004)
+   Copyright or  or Copr. Bio++ Development Team, (November 17, 2004)
 
    This software is a computer program whose purpose is to provide
    classes for sequences analysis.
@@ -41,6 +41,7 @@
 
 // From Utils:
 #include <Bpp/Text/TextTools.h>
+#include "AlphabetTools.h"
 
 #include <iostream>
 using namespace std;
@@ -50,138 +51,150 @@ using namespace bpp;
 
 RNY::RNY(const NucleicAlphabet& na) : nuclalph_(na)
 {
-  // Alphabet size definition
-  resize(351);
-  int i,j,k,l;
-
-  for (i = 0; i < 351; i++)
+  // Initialization:
+  vector<AlphabetState*> states;
+  for (int i = 0; i < 351; ++i)
   {
-    setState(i,AlphabetState(i,"",""));
+    states.push_back(new AlphabetState(i, TextTools::toString(i), ""));
   }
 
-  // Alphabet content definition
+  // Alphabet content definition:
 
-  // / changer pour ARN
+  string s1;
 
-  string s1 = "RCT-";
-  string s2 = "AGCT-";
+  if (AlphabetTools::isDNAAlphabet(&na))
+    s1 = "RCT-";
+  else 
+    s1 = "RCU-";
+  
+  string s2;
+ 
+  if (AlphabetTools::isDNAAlphabet(&na))
+    s2 = "AGCT-";
+  else
+    s2 = "AGCU-";
+  
   string s3 = "AGY-";
   string s = "   ";
 
 
   // NNN (0->35)
 
-  for (i = 0; i < 3; i++)
+  for (size_t i = 0; i < 3; ++i)
   {
-    for (j = 0; j < 4; j++)
+    for (size_t j = 0; j < 4; ++j)
     {
-      for (k = 0; k < 3; k++)
+      for (size_t k = 0; k < 3; ++k)
       {
-        l = i * 12 + j * 3 + k;
+        size_t l = i * 12 + j * 3 + k;
         s[0] = s1[i];
         s[1] = s2[j];
         s[2] = s3[k];
-        setState(l, AlphabetState(l,s,s));
+        states[l] = new AlphabetState(static_cast<int>(l), s, s);
       }
     }
   }
 
   // NN- (50->83)
 
-  for (i = 0; i < 3; i++)
+  for (size_t i = 0; i < 3; ++i)
   {
-    for (j = 0; j < 4; j++)
+    for (size_t j = 0; j < 4; ++j)
     {
-      l = 50 + 12 * i + j * 3;
+      size_t l = 50 + 12 * i + j * 3;
       s[0] = s1[i];
       s[1] = s2[j];
       s[2] = s3[3];
-      setState(l, AlphabetState(l,s,s));
+      states[l] = new AlphabetState(static_cast<int>(l), s, s);
     }
   }
 
   // N-N (100->126)
 
-  for (i = 0; i < 3; i++)
+  for (size_t i = 0; i < 3; ++i)
   {
-    for (k = 0; k < 3; k++)
+    for (size_t k = 0; k < 3; ++k)
     {
-      l = 100 + 12 * i + k;
+      size_t l = 100 + 12 * i + k;
       s[0] = s1[i];
       s[1] = s2[4];
       s[2] = s3[k];
-      setState(l, AlphabetState(l,s,s));
+      states[l] = new AlphabetState(static_cast<int>(l), s, s);
     }
   }
 
   // N-- (150->152)
 
-  for (i = 0; i < 3; i++)
+  for (size_t i = 0; i < 3; ++i)
   {
-    l = 150 + 12 * i;
+    size_t l = 150 + 12 * i;
     s[0] = s1[i];
     s[1] = s2[4];
     s[2] = s3[3];
-    setState(l, AlphabetState(l,s,s));
+    states[l] = new AlphabetState(static_cast<int>(l), s, s);
   }
 
   // -NN (200->211)
 
-  for (j = 0; j < 4; j++)
+  for (size_t j = 0; j < 4; ++j)
   {
-    for (k = 0; k < 3; k++)
+    for (size_t k = 0; k < 3; ++k)
     {
-      l = 200 + j * 3 + k;
+      size_t l = 200 + j * 3 + k;
       s[0] = s1[3];
       s[1] = s2[j];
       s[2] = s3[k];
-      setState(l, AlphabetState(l,s,s));
+      states[l] = new AlphabetState(static_cast<int>(l), s, s);
     }
   }
 
 
   // -N- (250->253)
 
-  for (j = 0; j < 4; j++)
+  for (size_t j = 0; j < 4; ++j)
   {
-    l = 250 + 3 * j;
+    size_t l = 250 + 3 * j;
     s[0] = s1[3];
     s[1] = s2[j];
     s[2] = s3[3];
-    setState(l, AlphabetState(l,s,s));
+    states[l] = new AlphabetState(static_cast<int>(l), s, s);
   }
 
   // --N (300->302)
 
-  for (k = 0; k < 3; k++)
+  for (size_t k = 0; k < 3; ++k)
   {
-    l = 300 + k;
+    size_t l = 300 + k;
     s[0] = s1[3];
     s[1] = s2[4];
     s[2] = s3[k];
-    setState(l, AlphabetState(l,s,s));
+    states[l] = new AlphabetState(static_cast<int>(l), s, s);
   }
 
 
   // --- (350)
 
-  l = 350;
   s[0] = s1[3];
   s[1] = s2[4];
   s[2] = s3[3];
-  setState(l, AlphabetState(l,s,s));
+  states[350] = new AlphabetState(350, s, s);
+
+  // Register all states:
+  for (size_t i = 0; i < states.size(); ++i)
+    registerState(states[i]);
 }
 
 /****************************************************************************************/
 
 vector<int> RNY::getAlias(int state) const throw (BadIntException)
 {
-  if (!isIntInAlphabet(state)) throw BadIntException(state, "RNY::getAlias(int): Specified base unknown.");
+  if (!isIntInAlphabet(state))
+    throw BadIntException(state, "RNY::getAlias(int): Specified base unknown.");
   vector<int> v;
 
   int qs = state / 50;
   int rs = state % 50;
-  int i,j,k;
+  int i, j, k;
 
   switch (qs)
   {
@@ -191,64 +204,64 @@ vector<int> RNY::getAlias(int state) const throw (BadIntException)
     break;
   case 1: // NN-
     v.resize(3);
-    for (k = 0; k < 3; k++)
+    for (k = 0; k < 3; ++k)
     {
-      v[k] = k + rs;
+      v[static_cast<size_t>(k)] = k + rs;
     }
     break;
   case 2: // N-N
     v.resize(4);
-    for (j = 0; j < 4; j++)
+    for (j = 0; j < 4; ++j)
     {
-      v[j] = 3 * j + rs;
+      v[static_cast<size_t>(j)] = 3 * j + rs;
     }
     break;
   case 3: // N--
     v.resize(12);
-    for (j = 0; j < 4; j++)
+    for (j = 0; j < 4; ++j)
     {
-      for (k = 0; k < 3; k++)
+      for (k = 0; k < 3; ++k)
       {
-        v[3 * j + k] = rs + 3 * j + k;
+        v[static_cast<size_t>(3 * j + k)] = rs + 3 * j + k;
       }
     }
     break;
   case 4: // -NN
     v.resize(3);
-    for (i = 0; i < 3; i++)
+    for (i = 0; i < 3; ++i)
     {
-      v[i] = 12 * i + rs;
+      v[static_cast<size_t>(i)] = 12 * i + rs;
     }
     break;
   case 5: // -N-
     v.resize(9);
-    for (i = 0; i < 3; i++)
+    for (i = 0; i < 3; ++i)
     {
-      for (k = 0; k < 3; k++)
+      for (k = 0; k < 3; ++k)
       {
-        v[3 * i + k] = rs + 12 * i + k;
+        v[static_cast<size_t>(3 * i + k)] = rs + 12 * i + k;
       }
     }
     break;
   case 6: // --N
     v.resize(12);
-    for (i = 0; i < 3; i++)
+    for (i = 0; i < 3; ++i)
     {
-      for (j = 0; j < 4; j++)
+      for (j = 0; j < 4; ++j)
       {
-        v[4 * i + j] = rs + 12 * i + 3 * j;
+        v[static_cast<size_t>(4 * i + j)] = rs + 12 * i + 3 * j;
       }
     }
     break;
   case 7: // ---
     v.resize(36);
-    for (i = 0; i < 3; i++)
+    for (i = 0; i < 3; ++i)
     {
-      for (j = 0; j < 4; j++)
+      for (j = 0; j < 4; ++j)
       {
-        for (k = 0; k < 3; k++)
+        for (k = 0; k < 3; ++k)
         {
-          v[12 * i + 3 * j + k] = 12 * i + 3 * j + k;
+          v[static_cast<size_t>(12 * i + 3 * j + k)] = 12 * i + 3 * j + k;
         }
       }
     }
@@ -266,7 +279,8 @@ const NucleicAlphabet& RNY::getLetterAlphabet() const
 
 vector<string> RNY::getAlias(const string& state) const throw (BadCharException)
 {
-  if (!isCharInAlphabet(state)) throw BadCharException(state, "RNY::getAlias(int): Specified base unknown.");
+  if (!isCharInAlphabet(state))
+    throw BadCharException(state, "RNY::getAlias(int): Specified base unknown.");
 
   vector<int> v = getAlias(charToInt(state));
   vector<string> s;
@@ -284,7 +298,7 @@ vector<string> RNY::getAlias(const string& state) const throw (BadCharException)
 
 string RNY::getRNY(const string& pos1, const string& pos2, const string& pos3) const throw (BadCharException)
 {
-   string tr;
+  string tr;
 
   if (pos1 == "A" || pos1 == "G")
     tr = "R";
@@ -293,7 +307,7 @@ string RNY::getRNY(const string& pos1, const string& pos2, const string& pos3) c
 
   tr += pos2;
 
-  if (pos3 == "T" || pos3 == "C")
+  if (pos3 == "T" || pos3 == "U" || pos3 == "C")
     tr += "Y";
   else
     tr += pos3;
@@ -306,9 +320,9 @@ string RNY::getRNY(const string& pos1, const string& pos2, const string& pos3) c
 /**************************************************************************************/
 int RNY::getRNY(int i, int j, int k, const Alphabet& alph) const throw (BadCharException)
 {
-  if (alph.getAlphabetType() != "DNA alphabet")
+  if (! AlphabetTools::isNucleicAlphabet(&alph))
   {
-    throw AlphabetException ("RNY::getRNY : Sequence must be DNA",
+    throw AlphabetException ("RNY::getRNY : Sequence must be Nucleic",
                              &alph);
   }
 
@@ -329,6 +343,7 @@ int RNY::getRNY(int i, int j, int k, const Alphabet& alph) const throw (BadCharE
     r += 1;
     break;
   case 'T':
+  case 'U':
     r += 2;
     break;
   case '-':
@@ -354,6 +369,7 @@ int RNY::getRNY(int i, int j, int k, const Alphabet& alph) const throw (BadCharE
     r += 2;
     break;
   case 'T':
+  case 'U':
     r += 3;
     break;
   case '-':
@@ -377,6 +393,7 @@ int RNY::getRNY(int i, int j, int k, const Alphabet& alph) const throw (BadCharE
     break;
   case 'C':
   case 'T':
+  case 'U':
     r += 2;
     break;
   case '-':
@@ -393,12 +410,12 @@ int RNY::getRNY(int i, int j, int k, const Alphabet& alph) const throw (BadCharE
 /****************************************************************************************/
 bool RNY::isGap(int state) const
 {
-  return state==350;
+  return state == 350;
 }
 
 bool RNY::containsGap(const string& state) const throw (BadCharException)
 {
-  return (state.find("-")!=string::npos);
+  return state.find("-") != string::npos;
 }
 
 bool RNY::isUnresolved(const string& state) const
@@ -408,15 +425,16 @@ bool RNY::isUnresolved(const string& state) const
 
 bool RNY::isUnresolved(int state) const
 {
-  return (state>=50 && state!=350);
+  return state >= 50 && state != 350;
 }
 
 /****************************************************************************************/
 
 int RNY::charToInt(const string& state) const throw (BadCharException)
 {
-  if (state.size() != 3) throw BadCharException(state, "RNY::charToInt", this);
-  else 
+  if (state.size() != 3)
+    throw BadCharException(state, "RNY::charToInt", this);
+  else
     return AbstractAlphabet::charToInt(state);
 }
 
@@ -425,12 +443,12 @@ int RNY::charToInt(const string& state) const throw (BadCharException)
 
 string RNY::intToChar(int state) const throw (BadIntException)
 {
-   unsigned int i,j,k,l;
-  for (i = 0; i < 3; i++)
+  int i, j, k, l;
+  for (i = 0; i < 3; ++i)
   {
-    for (j = 0; j < 4; j++)
+    for (j = 0; j < 4; ++j)
     {
-      for (k = 0; k < 3; k++)
+      for (k = 0; k < 3; ++k)
       {
         l = i * 12 + j * 3 + k;
         if (getState(l).getNum() == state)
@@ -441,9 +459,9 @@ string RNY::intToChar(int state) const throw (BadIntException)
 
   // NN- (50->83)
 
-  for (i = 0; i < 3; i++)
+  for (i = 0; i < 3; ++i)
   {
-    for (j = 0; j < 4; j++)
+    for (j = 0; j < 4; ++j)
     {
       l = 50 + 12 * i + j * 3;
       if (getState(l).getNum() == state)
@@ -453,9 +471,9 @@ string RNY::intToChar(int state) const throw (BadIntException)
 
   // N-N (100->126)
 
-  for (i = 0; i < 3; i++)
+  for (i = 0; i < 3; ++i)
   {
-    for (k = 0; k < 3; k++)
+    for (k = 0; k < 3; ++k)
     {
       l = 100 + 12 * i + k;
       if (getState(l).getNum() == state)
@@ -465,7 +483,7 @@ string RNY::intToChar(int state) const throw (BadIntException)
 
   // N-- (150->152)
 
-  for (i = 0; i < 3; i++)
+  for (i = 0; i < 3; ++i)
   {
     l = 150 + 12 * i;
     if (getState(l).getNum() == state)
@@ -474,9 +492,9 @@ string RNY::intToChar(int state) const throw (BadIntException)
 
   // -NN (200->211)
 
-  for (j = 0; j < 4; j++)
+  for (j = 0; j < 4; ++j)
   {
-    for (k = 0; k < 3; k++)
+    for (k = 0; k < 3; ++k)
     {
       l = 200 + j * 3 + k;
       if (getState(l).getNum() == state)
@@ -487,7 +505,7 @@ string RNY::intToChar(int state) const throw (BadIntException)
 
   // -N- (250->253)
 
-  for (j = 0; j < 4; j++)
+  for (j = 0; j < 4; ++j)
   {
     l = 250 + 3 * j;
     if (getState(l).getNum() == state)
@@ -496,7 +514,7 @@ string RNY::intToChar(int state) const throw (BadIntException)
 
   // --N (300->302)
 
-  for (k = 0; k < 3; k++)
+  for (k = 0; k < 3; ++k)
   {
     l = 300 + k;
     if (getState(l).getNum() == state)
