@@ -41,17 +41,15 @@
 #include "SequenceApplicationTools.h"
 #include "../Alphabet/BinaryAlphabet.h"
 #include "../Alphabet/DefaultAlphabet.h"
-#include "../Alphabet/EchinodermMitochondrialCodonAlphabet.h"
-#include "../Alphabet/InvertebrateMitochondrialCodonAlphabet.h"
-#include "../Alphabet/StandardCodonAlphabet.h"
-#include "../Alphabet/VertebrateMitochondrialCodonAlphabet.h"
-#include "../Alphabet/YeastMitochondrialCodonAlphabet.h"
+#include "../Alphabet/CodonAlphabet.h"
 #include "../Alphabet/AlphabetTools.h"
 #include "../GeneticCode/EchinodermMitochondrialGeneticCode.h"
 #include "../GeneticCode/InvertebrateMitochondrialGeneticCode.h"
 #include "../GeneticCode/StandardGeneticCode.h"
 #include "../GeneticCode/VertebrateMitochondrialGeneticCode.h"
 #include "../GeneticCode/YeastMitochondrialGeneticCode.h"
+#include "../GeneticCode/AscidianMitochondrialGeneticCode.h"
+#include "../GeneticCode/MoldMitochondrialGeneticCode.h"
 #include "../Io/BppOSequenceReaderFormat.h"
 #include "../Io/BppOAlignmentReaderFormat.h"
 #include "../Io/BppOSequenceWriterFormat.h"
@@ -64,6 +62,8 @@
 #include <Bpp/App/ApplicationTools.h>
 #include <Bpp/Text/TextTools.h>
 #include <Bpp/Text/KeyvalTools.h>
+#include <Bpp/App/NumCalcApplicationTools.h>
+#include <Bpp/Numeric/Random/RandomTools.h>
 
 using namespace bpp;
 using namespace std;
@@ -75,10 +75,11 @@ Alphabet* SequenceApplicationTools::getAlphabet(
   const string& suffix,
   bool suffixIsOptional,
   bool verbose,
-  bool allowGeneric) throw (Exception)
+  bool allowGeneric,
+  int warn) throw (Exception)
 {
   Alphabet* chars;
-  string alphtt = ApplicationTools::getStringParameter("alphabet", params, "DNA", suffix, suffixIsOptional);
+  string alphtt = ApplicationTools::getStringParameter("alphabet", params, "DNA", suffix, suffixIsOptional, warn);
 
   string alphabet = "";
   map<string, string> args;
@@ -105,17 +106,16 @@ Alphabet* SequenceApplicationTools::getAlphabet(
     flag = 2;
   }
 
-
   if (alphabet == "Binary")
     chars = new BinaryAlphabet();
   else if (alphabet == "DNA")
   {
-    bool mark = ApplicationTools::getBooleanParameter("bangAsGap", args, false, "", true, false);
+    bool mark = ApplicationTools::getBooleanParameter("bangAsGap", args, false, "", true, warn + 1);
     chars = new DNA(mark);
   }
   else if (alphabet == "RNA")
   {
-    bool mark = ApplicationTools::getBooleanParameter("bangAsGap", args, false, "", true, false);
+    bool mark = ApplicationTools::getBooleanParameter("bangAsGap", args, false, "", true, warn + 1);
     chars = new RNA(mark);
   }
   else if (alphabet == "Protein")
@@ -126,6 +126,8 @@ Alphabet* SequenceApplicationTools::getAlphabet(
   {
     if (args.find("letter") == args.end())
       throw Exception("Missing 'letter' argument in Codon :" + alphabet);
+    if (args.find("type") != args.end())
+      throw Exception("'type' argument in Codon is deprecated and has been superseded by the 'genetic_code' option.");
 
     string alphnDesc = ApplicationTools::getStringParameter("letter", args, "RNA");
     string alphn;
@@ -135,31 +137,19 @@ Alphabet* SequenceApplicationTools::getAlphabet(
     NucleicAlphabet* pnalph;
     if (alphn == "RNA")
     {
-      bool mark = ApplicationTools::getBooleanParameter("bangAsGap", alphnArgs, false, "", true, false);
+      bool mark = ApplicationTools::getBooleanParameter("bangAsGap", alphnArgs, false, "", true, warn + 1);
       pnalph = new RNA(mark);
     }
     else if (alphn == "DNA")
     {
-      bool mark = ApplicationTools::getBooleanParameter("bangAsGap", alphnArgs, false, "", true, false);
+      bool mark = ApplicationTools::getBooleanParameter("bangAsGap", alphnArgs, false, "", true, warn + 1);
       pnalph = new DNA(mark);
     }
     else
       throw Exception("Alphabet not known in Codon : " + alphn);
 
-    string type = ApplicationTools::getStringParameter("type", args, "Standard");
 
-    if (type == "EchinodermMitochondrial")
-      chars = new EchinodermMitochondrialCodonAlphabet(pnalph);
-    else if (type == "InvertebrateMitochondrial")
-      chars = new InvertebrateMitochondrialCodonAlphabet(pnalph);
-    else if (type == "Standard")
-      chars = new StandardCodonAlphabet(pnalph);
-    else if (type == "VertebrateMitochondrial")
-      chars = new VertebrateMitochondrialCodonAlphabet(pnalph);
-    else if (type == "YeastMitochondrial")
-      chars = new YeastMitochondrialCodonAlphabet(pnalph);
-    else
-      throw Exception("Unknown Alphabet : " + alphabet);
+    chars = new CodonAlphabet(pnalph);
     alphabet = alphabet + "(" + alphn + ")";
   }
   else
@@ -199,16 +189,20 @@ GeneticCode* SequenceApplicationTools::getGeneticCode(
   const string& description) throw (Exception)
 {
   GeneticCode* geneCode;
-  if (description.find("EchinodermMitochondrial") != string::npos)
+  if (description.find("EchinodermMitochondrial") != string::npos || description.find("9") != string::npos)
     geneCode = new EchinodermMitochondrialGeneticCode(alphabet);
-  else if (description.find("InvertebrateMitochondrial") != string::npos)
+  else if (description.find("InvertebrateMitochondrial") != string::npos || description.find("5") != string::npos)
     geneCode = new InvertebrateMitochondrialGeneticCode(alphabet);
-  else if (description.find("Standard") != string::npos)
+  else if (description.find("Standard") != string::npos || description.find("1") != string::npos)
     geneCode = new StandardGeneticCode(alphabet);
-  else if (description.find("VertebrateMitochondrial") != string::npos)
+  else if (description.find("VertebrateMitochondrial") != string::npos || description.find("2") != string::npos)
     geneCode = new VertebrateMitochondrialGeneticCode(alphabet);
-  else if (description.find("YeastMitochondrial") != string::npos)
+  else if (description.find("YeastMitochondrial") != string::npos || description.find("3") != string::npos)
     geneCode = new YeastMitochondrialGeneticCode(alphabet);
+  else if (description.find("AscidianMitochondrial") != string::npos || description.find("13") != string::npos)
+    geneCode = new AscidianMitochondrialGeneticCode(alphabet);
+  else if (description.find("MoldMitochondrial") != string::npos || description.find("4") != string::npos)
+    geneCode = new MoldMitochondrialGeneticCode(alphabet);
   else
     throw Exception("Unknown GeneticCode: " + description);
   return geneCode;
@@ -236,11 +230,12 @@ SequenceContainer* SequenceApplicationTools::getSequenceContainer(
   map<string, string>& params,
   const string& suffix,
   bool suffixIsOptional,
-  bool verbose)
+  bool verbose,
+  int warn)
 {
-  string sequenceFilePath = ApplicationTools::getAFilePath("input.sequence.file", params, true, true, suffix, suffixIsOptional);
-  string sequenceFormat = ApplicationTools::getStringParameter("input.sequence.format", params, "Fasta()", suffix, suffixIsOptional);
-  BppOSequenceReaderFormat bppoReader(verbose);
+  string sequenceFilePath = ApplicationTools::getAFilePath("input.sequence.file", params, true, true, suffix, suffixIsOptional, "none", warn);
+  string sequenceFormat = ApplicationTools::getStringParameter("input.sequence.format", params, "Fasta()", suffix, suffixIsOptional, warn);
+  BppOSequenceReaderFormat bppoReader(warn);
   auto_ptr<ISequence> iSeq(bppoReader.read(sequenceFormat));
   if (verbose)
   {
@@ -259,11 +254,12 @@ VectorSiteContainer* SequenceApplicationTools::getSiteContainer(
   map<string, string>& params,
   const string& suffix,
   bool suffixIsOptional,
-  bool verbose)
+  bool verbose,
+  int warn)
 {
-  string sequenceFilePath = ApplicationTools::getAFilePath("input.sequence.file", params, true, true, suffix, suffixIsOptional);
-  string sequenceFormat = ApplicationTools::getStringParameter("input.sequence.format", params, "Fasta()", suffix, suffixIsOptional);
-  BppOAlignmentReaderFormat bppoReader(verbose);
+  string sequenceFilePath = ApplicationTools::getAFilePath("input.sequence.file", params, true, true, suffix, suffixIsOptional, "none", warn);
+  string sequenceFormat = ApplicationTools::getStringParameter("input.sequence.format", params, "Fasta()", suffix, suffixIsOptional, warn);
+  BppOAlignmentReaderFormat bppoReader(warn);
   auto_ptr<IAlignment> iAln(bppoReader.read(sequenceFormat));
   map<string, string> args(bppoReader.getUnparsedArguments());
   if (verbose)
@@ -298,11 +294,13 @@ VectorSiteContainer* SequenceApplicationTools::getSiteContainer(
   else
     sites = sites2;
 
+
+  
   // Look for site selection:
   if (iAln->getFormatName() == "MASE file")
   {
     // getting site set:
-    string siteSet = ApplicationTools::getStringParameter("siteSelection", args, "none", suffix, suffixIsOptional, false);
+    string siteSet = ApplicationTools::getStringParameter("siteSelection", args, "none", suffix, suffixIsOptional, warn + 1);
     if (siteSet != "none")
     {
       VectorSiteContainer* selectedSites;
@@ -324,6 +322,62 @@ VectorSiteContainer* SequenceApplicationTools::getSiteContainer(
       sites = selectedSites;
     }
   }
+  else
+  {
+    // getting site set:
+    size_t nbSites = sites->getNumberOfSites();
+    
+    string siteSet = ApplicationTools::getStringParameter("input.site.selection", params, "none", suffix, suffixIsOptional, warn + 1);
+
+    VectorSiteContainer* selectedSites=0;
+    if (siteSet != "none")
+    {
+      vector<size_t> vSite;
+      try {
+        vector<int> vSite1 = NumCalcApplicationTools::seqFromString(siteSet);
+        for (size_t i = 0; i < vSite1.size(); ++i){
+          int x = (vSite1[i] >= 0 ? vSite1[i] : static_cast<int>(nbSites) + vSite1[i]);
+          if (x >= 0)
+            vSite.push_back(static_cast<size_t>(x-1));
+          else
+            throw Exception("SequenceApplicationTools::getSiteContainer(). Incorrect negative index: " + TextTools::toString(x));
+        }
+        selectedSites = dynamic_cast<VectorSiteContainer*>(SiteContainerTools::getSelectedSites(*sites, vSite));
+      }
+      catch (Exception& e)
+      {
+        string seln;
+        map<string, string> selArgs;
+        KeyvalTools::parseProcedure(siteSet, seln, selArgs);
+        if (seln == "Sample")
+        {
+          size_t n = ApplicationTools::getParameter<size_t>("n", selArgs, nbSites, "", true, warn + 1);
+          bool replace = ApplicationTools::getBooleanParameter("replace", selArgs, false, "", true, warn + 1);
+
+          vSite.resize(n);
+          vector<size_t> vPos;
+          for (size_t p = 0; p < nbSites; ++p)
+            vPos.push_back(p);
+          
+          RandomTools::getSample(vPos, vSite, replace);
+
+          selectedSites = dynamic_cast<VectorSiteContainer*>(SiteContainerTools::getSelectedSites(*sites, vSite));
+          if (replace)
+            selectedSites->reindexSites();
+        }
+      }
+
+      if (verbose)
+        ApplicationTools::displayResult("Selected sites", TextTools::toString(siteSet));
+
+      if (selectedSites && (selectedSites->getNumberOfSites() == 0))
+      {
+        throw Exception("Site set '" + siteSet + "' is empty.");
+      }
+      delete sites;
+      sites = selectedSites;
+    }
+  }
   return sites;
 }
 
@@ -335,19 +389,20 @@ VectorSiteContainer* SequenceApplicationTools::getSitesToAnalyse(
   string suffix,
   bool suffixIsOptional,
   bool gapAsUnknown,
-  bool verbose)
+  bool verbose,
+  int warn)
 {
   // Fully resolved sites, i.e. without jokers and gaps:
   SiteContainer* sitesToAnalyse;
   VectorSiteContainer* sitesToAnalyse2;
 
-  string option = ApplicationTools::getStringParameter("input.sequence.sites_to_use", params, "complete", suffix, suffixIsOptional);
+  string option = ApplicationTools::getStringParameter("input.sequence.sites_to_use", params, "complete", suffix, suffixIsOptional, warn);
   if (verbose)
     ApplicationTools::displayResult("Sites to use", option);
   if (option == "all")
   {
     sitesToAnalyse = new VectorSiteContainer(allSites);
-    string maxGapOption = ApplicationTools::getStringParameter("input.sequence.max_gap_allowed", params, "100%", suffix, suffixIsOptional);
+    string maxGapOption = ApplicationTools::getStringParameter("input.sequence.max_gap_allowed", params, "100%", suffix, suffixIsOptional, warn);
 
     if (maxGapOption[maxGapOption.size() - 1] == '%')
     {
@@ -390,9 +445,9 @@ VectorSiteContainer* SequenceApplicationTools::getSitesToAnalyse(
       }
     }
 
-    string maxUnresolvedOption = ApplicationTools::getStringParameter("input.sequence.max_unresolved_allowed", params, "100%", suffix, suffixIsOptional);
+    string maxUnresolvedOption = ApplicationTools::getStringParameter("input.sequence.max_unresolved_allowed", params, "100%", suffix, suffixIsOptional, warn);
 
-    size_t sAlph = sitesToAnalyse->getAlphabet()->getSize();
+    int sAlph = static_cast<int>(sitesToAnalyse->getAlphabet()->getSize());
 
     if (maxUnresolvedOption[maxUnresolvedOption.size() - 1] == '%')
     {
@@ -408,7 +463,7 @@ VectorSiteContainer* SequenceApplicationTools::getSitesToAnalyse(
           map<int, double> freq;
           SiteTools::getFrequencies(sitesToAnalyse->getSite(i - 1), freq);
           double x = 0;
-          for (unsigned int l = 0; l < sAlph; l++)
+          for (int l = 0; l < sAlph; ++l)
           {
             x += freq[l];
           }
@@ -434,7 +489,7 @@ VectorSiteContainer* SequenceApplicationTools::getSitesToAnalyse(
           map<int, size_t> counts;
           SiteTools::getCounts(sitesToAnalyse->getSite(i - 1), counts);
           size_t x = 0;
-          for (int l = 0; l < static_cast<int>(sAlph); l++)
+          for (int l = 0; l < sAlph; l++)
           {
             x += counts[l];
           }
@@ -471,15 +526,18 @@ VectorSiteContainer* SequenceApplicationTools::getSitesToAnalyse(
     throw Exception("Option '" + option + "' unknown in parameter 'sequence.sites_to_use'.");
   }
 
-  if (AlphabetTools::isCodonAlphabet(sitesToAnalyse->getAlphabet()))
+  const CodonAlphabet* ca = dynamic_cast<const CodonAlphabet*>(sitesToAnalyse->getAlphabet());
+  if (ca)
   {
-    option = ApplicationTools::getStringParameter("input.sequence.remove_stop_codons", params, "no", suffix, true);
+    option = ApplicationTools::getStringParameter("input.sequence.remove_stop_codons", params, "no", suffix, true, warn);
     if ((option != "") && verbose)
       ApplicationTools::displayResult("Remove Stop Codons", option);
 
     if (option == "yes")
     {
-      sitesToAnalyse2 = dynamic_cast<VectorSiteContainer*>(SiteContainerTools::removeStopCodonSites(*sitesToAnalyse));
+      string codeDesc = ApplicationTools::getStringParameter("genetic_code", params, "Standard", "", true, warn);
+      auto_ptr<GeneticCode> gCode(getGeneticCode(ca->getNucleicAlphabet(), codeDesc));
+      sitesToAnalyse2 = dynamic_cast<VectorSiteContainer*>(SiteContainerTools::removeStopCodonSites(*sitesToAnalyse, *gCode));
       delete sitesToAnalyse;
     }
     else
@@ -497,11 +555,12 @@ void SequenceApplicationTools::writeSequenceFile(
   const SequenceContainer& sequences,
   map<string, string>& params,
   const string& suffix,
-  bool verbose)
+  bool verbose,
+  int warn)
 {
-  string sequenceFilePath = ApplicationTools::getAFilePath("output.sequence.file", params, true, false, suffix, false);
-  string sequenceFormat   = ApplicationTools::getStringParameter("output.sequence.format", params, "Fasta", suffix, false, true);
-  BppOSequenceWriterFormat bppoWriter(verbose);
+  string sequenceFilePath = ApplicationTools::getAFilePath("output.sequence.file", params, true, false, suffix, false, "none", warn);
+  string sequenceFormat   = ApplicationTools::getStringParameter("output.sequence.format", params, "Fasta", suffix, false, warn);
+  BppOSequenceWriterFormat bppoWriter(warn);
   auto_ptr<OSequence> oSeq(bppoWriter.read(sequenceFormat));
   if (verbose)
   {
@@ -519,11 +578,12 @@ void SequenceApplicationTools::writeAlignmentFile(
   const SiteContainer& sequences,
   map<string, string>& params,
   const string& suffix,
-  bool verbose)
+  bool verbose,
+  int warn)
 {
-  string sequenceFilePath = ApplicationTools::getAFilePath("output.sequence.file", params, true, false, suffix, false);
-  string sequenceFormat   = ApplicationTools::getStringParameter("output.sequence.format", params, "Fasta", suffix, false, true);
-  BppOAlignmentWriterFormat bppoWriter(verbose);
+  string sequenceFilePath = ApplicationTools::getAFilePath("output.sequence.file", params, true, false, suffix, false, "none", warn);
+  string sequenceFormat   = ApplicationTools::getStringParameter("output.sequence.format", params, "Fasta", suffix, false, warn);
+  BppOAlignmentWriterFormat bppoWriter(warn);
   auto_ptr<OAlignment> oAln(bppoWriter.read(sequenceFormat));
   if (verbose)
   {
