@@ -77,8 +77,8 @@ Sequence* SequenceTools::subseq(const Sequence& sequence, size_t begin, size_t e
   vector<int> temp(sequence.getContent());
 
   // Truncate sequence
-  temp.erase(temp.begin() + end + 1, temp.end());
-  temp.erase(temp.begin(), temp.begin() + begin);
+  temp.erase(temp.begin() + static_cast<ptrdiff_t>(end + 1), temp.end());
+  temp.erase(temp.begin(), temp.begin() + static_cast<ptrdiff_t>(begin));
 
   // New sequence creation
   return new BasicSequence(sequence.getName(), temp, sequence.getComments(), sequence.getAlphabet());
@@ -181,7 +181,7 @@ Sequence* SequenceTools::reverseTranscript(const Sequence& sequence) throw (Alph
 Sequence& SequenceTools::invert(Sequence& seq)
 {
   size_t seq_size = seq.size(); // store seq size for efficiency
-  unsigned int tmp_state = 0; // to store one state when swapping positions
+  int tmp_state = 0; // to store one state when swapping positions
   size_t j = seq_size; // symetric position iterator from sequence end
   for (size_t i = 0; i < seq_size / 2; i++)
   {
@@ -304,6 +304,22 @@ size_t SequenceTools::getNumberOfCompleteSites(const Sequence& seq)
 
 /******************************************************************************/
 
+Sequence* SequenceTools::getSequenceWithCompleteSites(const Sequence& seq)
+{
+  const Alphabet* alpha = seq.getAlphabet();
+  vector<int> content;
+  for (size_t i = 0; i < seq.size(); i++)
+  {
+    if (!(alpha->isGap(seq[i]) || alpha->isUnresolved(seq[i])))
+      content.push_back(seq[i]);
+  }
+  Sequence* newSeq = dynamic_cast<Sequence*>(seq.clone());
+  newSeq->setContent(content);
+  return newSeq;
+}
+
+/******************************************************************************/
+
 size_t SequenceTools::getNumberOfUnresolvedSites(const Sequence& seq)
 {
   size_t count = 0;
@@ -346,7 +362,7 @@ void SequenceTools::removeGaps(Sequence& seq)
 
 /******************************************************************************/
 
-Sequence* SequenceTools::getSequenceWithoutStops(const Sequence& seq) throw (Exception)
+Sequence* SequenceTools::getSequenceWithoutStops(const Sequence& seq, const GeneticCode& gCode) throw (Exception)
 {
   const CodonAlphabet* calpha = dynamic_cast<const CodonAlphabet*>(seq.getAlphabet());
   if (!calpha)
@@ -354,7 +370,7 @@ Sequence* SequenceTools::getSequenceWithoutStops(const Sequence& seq) throw (Exc
   vector<int> content;
   for (size_t i = 0; i < seq.size(); i++)
   {
-    if (!calpha->isStop(seq[i]))
+    if (!gCode.isStop(seq[i]))
       content.push_back(seq[i]);
   }
   Sequence* newSeq = dynamic_cast<Sequence*>(seq.clone());
@@ -364,21 +380,21 @@ Sequence* SequenceTools::getSequenceWithoutStops(const Sequence& seq) throw (Exc
 
 /******************************************************************************/
 
-void SequenceTools::removeStops(Sequence& seq) throw (Exception)
+void SequenceTools::removeStops(Sequence& seq, const GeneticCode& gCode) throw (Exception)
 {
   const CodonAlphabet* calpha = dynamic_cast<const CodonAlphabet*>(seq.getAlphabet());
   if (!calpha)
     throw Exception("SequenceTools::removeStops. Input sequence should have a codon alphabet.");
   for (size_t i = seq.size(); i > 0; --i)
   {
-    if (calpha->isStop(seq[i - 1]))
+    if (gCode.isStop(seq[i - 1]))
       seq.deleteElement(i - 1);
   }
 }
 
 /******************************************************************************/
 
-void SequenceTools::replaceStopsWithGaps(Sequence& seq) throw (Exception)
+void SequenceTools::replaceStopsWithGaps(Sequence& seq, const GeneticCode& gCode) throw (Exception)
 {
   const CodonAlphabet* calpha = dynamic_cast<const CodonAlphabet*>(seq.getAlphabet());
   if (!calpha)
@@ -386,7 +402,7 @@ void SequenceTools::replaceStopsWithGaps(Sequence& seq) throw (Exception)
   int gap = calpha->getGapCharacterCode();
   for (size_t i = 0; i < seq.size(); ++i)
   {
-    if (calpha->isStop(seq[i]))
+    if (gCode.isStop(seq[i]))
       seq.setElement(i, gap);
   }
 }
@@ -568,7 +584,7 @@ Sequence* SequenceTools::RNYslice(const Sequence& seq, int ph) throw (AlphabetEx
     throw Exception("Bad phase for RNYSlice: " + TextTools::toString(ph) + ". Should be between 1 and 3.");
 
   size_t s = seq.size();
-  size_t n = (s - ph + 3) / 3;
+  size_t n = (s - static_cast<size_t>(ph) + 3) / 3;
 
   vector<int> content(n);
 
@@ -577,7 +593,7 @@ Sequence* SequenceTools::RNYslice(const Sequence& seq, int ph) throw (AlphabetEx
 
   for (size_t i = 0; i < n; i++)
   {
-    j = i * 3 + ph - 1;
+    j = i * 3 + static_cast<size_t>(ph) - 1;
 
     if (j == 0)
       content[i] = _RNY.getRNY(tir, seq[0], seq[1], *seq.getAlphabet());
@@ -636,27 +652,27 @@ Sequence* SequenceTools::RNYslice(const Sequence& seq) throw (AlphabetException)
 /******************************************************************************/
 
 
-void SequenceTools::getCDS(Sequence& sequence, bool checkInit, bool checkStop, bool includeInit, bool includeStop)
+void SequenceTools::getCDS(Sequence& sequence, const GeneticCode& gCode, bool checkInit, bool checkStop, bool includeInit, bool includeStop)
 {
   const CodonAlphabet* alphabet = dynamic_cast<const CodonAlphabet*>(sequence.getAlphabet());
   if (!alphabet)
     throw AlphabetException("SequenceTools::getCDS. Sequence is not a codon sequence.");
   if (checkInit)
   {
-    unsigned int i;
-    for (i = 0; i < sequence.size() && !alphabet->isInit(sequence[i]); ++i)
+    size_t i;
+    for (i = 0; i < sequence.size() && !gCode.isStart(sequence[i]); ++i)
     {}
-    for (unsigned int j = 0; includeInit ? j < i : j <= i; ++j)
+    for (size_t j = 0; includeInit ? j < i : j <= i; ++j)
     {
       sequence.deleteElement(j);
     }
   }
   if (checkStop)
   {
-    unsigned int i;
-    for (i = 0; i < sequence.size() && !alphabet->isStop(sequence[i]); ++i)
+    size_t i;
+    for (i = 0; i < sequence.size() && !gCode.isStop(sequence[i]); ++i)
     {}
-    for (unsigned int j = includeStop ? i + 1 : i; j < sequence.size(); ++j)
+    for (size_t j = includeStop ? i + 1 : i; j < sequence.size(); ++j)
     {
       sequence.deleteElement(j);
     }
@@ -696,4 +712,16 @@ size_t SequenceTools::findFirstOf(const Sequence& seq, const Sequence& motif, bo
 }
 
 /******************************************************************************/
+  
+Sequence* SequenceTools::getRandomSequence(const Alphabet* alphabet, size_t length)
+{
+  int s = static_cast<int>(alphabet->getSize());
+  vector<int> content(length);
+  for (size_t i = 0; i < length; ++i) {
+    content[i] = RandomTools::giveIntRandomNumberBetweenZeroAndEntry(s);
+  }
+  return new BasicSequence("random", content, alphabet);
+}
+
+/******************************************************************************/
 
