@@ -86,11 +86,6 @@ public:
    */
   static bool hasUnknown(const Site& site);
 
-  /**
-   * @param site A site.
-   * @return True if the site contains a Stop Codon, when the alphabet  is a CodonAlphabet.
-   */
-  static bool hasStopCodon(const Site& site);
   /**
    * @param site A site.
    * @return True if the site contains no gap and no unknown characters.
